from ...app import Kard
from ... import kard

class KardContacts:
    def __init__(self, kard: Kard):
        self.app = kard


    def getCompleteData(self):
        query = "query androidListContacts { me { contacts(kardersOnly: true) { "\
                    "identifier status user { __typename avatar { url } hasBankAccount id firstName lastName username } "\
                "} }}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('contacts'):
            raise kard.exceptions.GraphQLException('Expected "me.contacts" to be present. Got: ' + str(response))

        return response['me']['contacts']

    def getAll(self):
        return self.getCompleteData()
    
    def get(self, identifier: str|int):
        cId = str(identifier)

        for contact in self.getAll():
            if contact['identifier'] == cId:
                return contact

        raise kard.exceptions.ContactNotFoundError("Contact %s not found" % cId)

    def getAllParents(self):
        contacts = []
        for contact in self.getAll():
            if contact['user']['__typename'] == 'ParentUser':
                contacts.append(contact)
        
        return contacts

    def getAllAdults(self):
        contacts = []
        for contact in self.getAll():
            if contact['user']['__typename'] == 'AdultUser':
                contacts.append(contact)
        
        return contacts

    def getAllTeens(self):
        contacts = []
        for contact in self.getAll():
            if contact['user']['__typename'] == 'TeenUser':
                contacts.append(contact)
        
        return contacts



from ...app import Kard
from ... import kard

class KardBeneficiary:
    def __init__(self, kard: Kard):
        self.app = kard

    def _new(self, beneficiaryId: str, **kwargs):
        beneficiary = KardBeneficiary(self.app)
        beneficiary._setId(beneficiaryId)

        beneficiary._name = kwargs.get('fullName')
        beneficiary._bankName = kwargs.get('bankName')
        beneficiary._bankLogo = kwargs.get('bankLogo')
        beneficiary._bankSurname = kwargs.get('surname')
        beneficiary._iban = kwargs.get('iban')
        beneficiary._bic = kwargs.get('bic')
        beneficiary.deleted = False

        return beneficiary

    def _setId(self, beneficiaryId: str):
        self._id = beneficiaryId

    def _getId(self):
        return self._id

    @property
    def id(self):
        return self._getId()
    
    @property
    def name(self):
        return self._name

    @property
    def bankName(self):
        return self._bankName

    @property
    def bankSurname(self):
        return self._bankSurname
    
    @property
    def bankLogo(self):
        return self._bankLogo

    @property
    def iban(self):
        return self._iban
    
    @property
    def bic(self):
        return self._bic


    def setBeneficiaryName(self, name: str):
        return self.editBeneficiary(name, self.bankSurname, self.iban, self.bic)

    def setBeneficiaryBankName(self, bankName: str):
        return self.editBeneficiary(self.name, bankName, self.iban, self.bic)
    
    def setBeneficiaryIban(self, iban: str):
        return self.editBeneficiary(self.name, self.bankSurname, iban, self.bic)
    
    def setBeneficiaryBic(self, bic: str):
        return self.editBeneficiary(self.name, self.bankSurname, self.iban, bic)

    def editBeneficiary(self, name: str, bankName: str, iban: str, bic: str):
        query = "mutation androidUpdateBeneficiary("\
                    "$iban: FrenchIban!, $bic: Bic!, $displayName: BankInfoDisplayName!, "\
                    "$surname: String, $beneficiary_id: ID!"\
                ") { "\
                        "updateBeneficiary(input: { "\
                            "beneficiaryInfo: { iban: $iban, bic: $bic, displayName: $displayName, surname: $surname }, "\
                            "beneficiaryId: $beneficiary_id "\
                        "}) { "\
                            "errors { message path extensions { code } } "\
                        "}"\
                    "}"
        variables = {
            "iban": iban,
            "surname": bankName,
            "beneficiary_id": self.id,
            "bic": bic,
            "displayName": name
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('updateBeneficiary'):
            raise kard.exceptions.GraphQLException('Expected "updateBeneficiary" to be present. Got: ' + str(response))
        
        if response['updateBeneficiary'].get('errors'):
            for error in response['updateBeneficiary']['errors']:
                if error['message'] == 'Variable $bic of type Bic! was provided invalid value':
                    raise ValueError('Invalid BIC %s provided' % bic)
                elif error['message'] == 'Variable $iban of type FrenchIban! was provided invalid value':
                    raise ValueError('Invalid IBAN %s provided' % iban)
                elif error['message'] == 'Variable $surname of type String was provided invalid value':
                    raise TypeError('Invalid surname %s provided. Must be of type str' % bankName)
        
            raise kard.exceptions.GraphQLException('Expected "updateBeneficiary.errors" to be empty. Got: ' + str(response))

        self._name = name
        self._bankName = bankName
        self._bankSurname = bankName
        self._iban = iban
        self._bic = bic

        return True

    def delete(self):
        query = "mutation androidDeleteBeneficiary($beneficiary_id: ID!) { "\
                    "deleteBeneficiary(input: { beneficiaryId: $beneficiary_id }) { "\
                        "errors { message path } "\
                    "}"\
                "}"
        variables = {
            "beneficiary_id": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('deleteBeneficiary'):
            raise kard.exceptions.GraphQLException('Expected "deleteBeneficiary" to be present. Got: ' + str(response))
        
        if response['deleteBeneficiary'].get('errors'):
            raise kard.exceptions.GraphQLException('Expected "deleteBeneficiary.errors" to be empty. Got: ' + str(response))
        
        self.deleted = True
        return True

    def sendWireTransfer(self, amount: float, currency: str='EUR', reason: str=''):
        query = "mutation androidSendWireTransfer($amount: AmountInput!, $id: ID! $reason: WireTransferReason!) { "\
                    "sendWireTransfer(input: {beneficiaryId: $id, amount: $amount, reason: $reason}) { "\
                        "errors { message path extensions { code }} "\
                    "}"\
                "}"
        variables = {
            "amount": {
                "value": amount,
                "currency": currency
            },
            "id": self.id,
            "reason": reason
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('sendWireTransfer'):
            raise kard.exceptions.GraphQLException('Expected "sendWireTransfer" to be present. Got: ' + str(response))

        if response['sendWireTransfer'].get('errors'):
            for error in response['sendWireTransfer']['errors']:
                errCode = error.get('extensions', {}).get('code')
                if errCode == 'INSUFFICIENT_BALANCE':
                    raise kard.exceptions.InsufficientFundsException('Insufficient funds to send wire transfer')
                elif errCode == 'AMOUNT_TOO_SMALL':
                    raise kard.exceptions.WireTransferInvalidAmount(error['message'])

            raise kard.exceptions.GraphQLException('Expected "sendWireTransfer.errors" to be empty. Got: ' + str(response))

        return True

    def sendMoney(self, **kwargs):
        return self.sendWireTransfer(**kwargs)

    def transfer(self, **kwargs):
        return self.sendWireTransfer(**kwargs)



from ...app import Kard
from ... import kard

class KardContact:
    def __init__(self, kard: Kard):
        self.app = kard

    def _new(self, contactId: str, **kwargs):
        contact = KardContact(self.app)
        contact._setId(contactId)

        contact._type = kwargs.get('type')
        contact._status = kwargs.get('status')
        contact._hasBankAccount = kwargs.get('hasBankAccount')
        contact._firstName = kwargs.get('firstName')
        contact._lastName = kwargs.get('lastName')
        contact._username = kwargs.get('username')
        contact._avatar = kwargs.get('avatar')

        return contact

    def _setId(self, contactId: str):
        self._id = contactId
    
    def _getId(self):
        return self._id
    
        



from ...app import Kard
from ... import kard

class KardKYC:
    def __init__(self, kard: Kard):
        self.app = kard

    
    def getCompleteData(self):
        query = "query androidKyc { me { kyc { ... Me_KycParts } }}\n\n"\
                "fragment Me_KycParts on Kyc { "\
                    "required deadline globalStatus fundsOrigin { status value rejectionReason } "\
                    "identityVerification { status url rejectionReason score } "\
                    "proofOfAddress { status files { contentType url ... on Image { width height } } rejectionReason } "\
                    "canStart lastRejectedIdentityVerification { status url rejectionReason score} nbOfRejectedIdentityVerifications"\
                "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('kyc', {}):
            raise kard.exceptions.GraphQLException('Expected "me.kyc" to be present. Got: ' + str(response))

        return response['me']['kyc']

    @property
    def isCompleted(self):
        return self.getCompletedStatus() == 'APPROVED'

    @property
    def deadline(self):
        return self.getDeadline()

    @property
    def fundsOrigin(self):
        return self.getFundsOrigin()

    @property
    def proofsOfAddress(self):
        return self.getProofsOfAddress()


    def getCompletedStatus(self):
        query = "query androidKyc { me { kyc { globalStatus } }}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('kyc', {}):
            raise kard.exceptions.GraphQLException('Expected "me.kyc" to be present. Got: ' + str(response))

        return response['me']['kyc']['globalStatus']

    def getDeadline(self):
        query = "query androidKyc { me { kyc { deadline } }}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('kyc', {}):
            raise kard.exceptions.GraphQLException('Expected "me.kyc" to be present. Got: ' + str(response))

        return response['me']['kyc']['deadline']

    def getFundsOrigin(self):
        query = "query androidKyc { me { kyc { fundsOrigin { status value rejectionReason } }}}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('kyc', {}):
            raise kard.exceptions.GraphQLException('Expected "me.kyc" to be present. Got: ' + str(response))

        return response['me']['kyc']['fundsOrigin']['value']

    def getProofsOfAddress(self):
        query = "query androidKyc { me { kyc { "\
                    "proofOfAddress { status files { contentType url ... on Image { width height } } rejectionReason } "\
                "}}}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('kyc', {}):
            raise kard.exceptions.GraphQLException('Expected "me.kyc" to be present. Got: ' + str(response))

        return response['me']['kyc']['proofOfAddress']['files']




from ...app import Kard
from ... import kard

class KardSubscription:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
				"fragment Me_MeParts on Me { subscription {"\
                    "id status cancelledAt cancellationReason nextBilling { date amount { value } } "\
                    "plan { __typename id periodUnit name providerId price { value } } } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('subscription'):
            raise kard.exceptions.GraphQLException('Expected "me.subscription" to be present. Got: ' + str(response))

        return response['me']['subscription']

    @property
    def id(self):
        return self.getId()

    @property
    def isActive(self):
        status = self.getStatus()
        return status == 'ACTIVE'

    @property
    def nextBillingDate(self):
        return self.getNextBilling()['date']
    
    @property
    def nextBillingAmount(self):
        nextBilling = self.getNextBilling()
        if nextBilling['amount']:
            return nextBilling['amount']['value']
        return None # Should return None or 0?

    @property
    def planName(self):
        return self.getPlan()['name']

    @property
    def planId(self):
        return self.getPlan()['id']
    
    @property
    def planPrice(self):
        return self.getPlan()['price']['value']

    @property
    def price(self):
        return self.planPrice

    @property
    def isPaidMonthly(self):
        period = self.getPlan()['periodUnit']
        return period == 'MONTH'

    @property
    def isPaidYearly(self):
        period = self.getPlan()['periodUnit']
        return period == 'YEAR'


    def getId(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
                "fragment Me_MeParts on Me { subscription { id } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('subscription', {}).get('id'):
            raise kard.exceptions.GraphQLException('Expected "me.subscription.id" to be present. Got: ' + str(response))

        return response['me']['subscription']['id']

    def getStatus(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
                "fragment Me_MeParts on Me { subscription { status } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('subscription', {}).get('status'):
            raise kard.exceptions.GraphQLException('Expected "me.subscription.status" to be present. Got: ' + str(response))

        return response['me']['subscription']['status']

    def getNextBilling(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
                "fragment Me_MeParts on Me { subscription { nextBilling { date amount { value } } } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('subscription', {}).get('nextBilling'):
            raise kard.exceptions.GraphQLException('Expected "me.subscription.nextBilling" to be present. Got: ' + str(response))

        return response['me']['subscription']['nextBilling']
	
    def getPlan(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
                "fragment Me_MeParts on Me { subscription { plan { __typename id periodUnit name providerId price { value } } } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('subscription', {}).get('plan'):
            raise kard.exceptions.GraphQLException('Expected "me.subscription.plan" to be present. Got: ' + str(response))

        return response['me']['subscription']['plan']




from ...app import Kard
from ... import kard

class KardCashback:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Transaction_TransactionCashbackParts on Transaction { "\
                "id title amount { value currency { symbol } } image { id url } processedAt category { name color image { url } }"\
                " ... on CashbackTransaction { cashback { status brandLogo brandName sourceTransaction { "\
                    "id title image { id url } category { name color image { url } } amount { value currency { symbol } } } } }"\
            "}\n\n"\
            "fragment Me_MeParts on Me { "\
                "cashbackEnabled cashbackWallet { "\
                    "id balance { value currency { symbol isoCode } } "\
                    "amountEarned { value currency { symbol isoCode } } "\
                    "transactions(first: 10, order: CREATED, direction: DESC) { "\
                        "pageInfo { endCursor hasNextPage } "\
                        "nodes { ... Transaction_TransactionCashbackParts } "\
                    "} "\
                "}"\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('cashbackWallet'):
            raise kard.exceptions.GraphQLException('Expected "me.cashbackWallet" to be present. Got: ' + str(response))

        return response['me']

    @property
    def walletId(self):
        return self.getWalletId()

    @property
    def isEnabled(self):
        return self.isEnabled()

    @property
    def balance(self):
        return self.getBalance()
    
    @property
    def amountEarned(self):
        return self.getAmountEarned()
    
    @property
    def totalEarned(self):
        return self.amountEarned
    
    @property
    def earned(self):
        return self.amountEarned

    @property
    def transactions(self):
        return self.getTransactions()

    def getWalletId(self):
        query = "query androidMe { me { cashbackWallet { id } } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('cashbackWallet', {}).get('id'):
            raise kard.exceptions.GraphQLException('Expected "me.cashbackWallet.id" to be present. Got: ' + str(response))
        
        return response['me']['cashbackWallet']['id']

    def isEnabled(self):
        query = "query androidMe { me { cashbackEnabled } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('cashbackEnabled'):
            raise kard.exceptions.GraphQLException('Expected "me.cashbackEnabled" to be present. Got: ' + str(response))
        
        return response['me']['cashbackEnabled']

    def getBalance(self):
        query = "query androidMe { me { cashbackWallet { balance { value currency { symbol isoCode } } } } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('cashbackWallet', {}).get('balance'):
            raise kard.exceptions.GraphQLException('Expected "me.cashbackWallet.balance" to be present. Got: ' + str(response))
        
        return response['me']['cashbackWallet']['balance']

    def getAmountEarned(self):
        query = "query androidMe { me { cashbackWallet { amountEarned { value currency { symbol isoCode } } } } }"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('cashbackWallet', {}).get('amountEarned'):
            raise kard.exceptions.GraphQLException('Expected "me.cashbackWallet.amountEarned" to be present. Got: ' + str(response))
        
        return response['me']['cashbackWallet']['amountEarned']

    def getTransactions(self, first: int=20, after: str=None):
        query = "query androidCashbackTransactions($first: Int, $after: String) { "\
                "me { "\
                    "cashbackWallet { "\
                        "transactions(first: $first, after: $after, order: CREATED, direction: DESC) { "\
                            "pageInfo { endCursor hasNextPage } nodes { ... Transaction_TransactionCashbackParts } "\
                        "} "\
                    "} "\
                "}"\
            "}\n\n"\
            "fragment Transaction_TransactionCashbackParts on Transaction { "\
                "id title amount { value currency { symbol } } image { id url } processedAt "\
                "category { name color image { url } } ... on CashbackTransaction { "\
                    "cashback { "\
                        "status brandLogo brandName sourceTransaction { "\
                            "id title image { id url } category { name color image { url } } amount { value currency { symbol } } "\
                        "} "\
                    "} "\
                "}"\
            "}"
        variables = {
            "first": 10,
            "after": None
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('cashbackWallet', {}).get('transactions'):
            raise kard.exceptions.GraphQLException('Expected "me.cashbackWallet.transactions" to be present. Got: ' + str(response))

        return response['me']['cashbackWallet']['transactions']
    
    # def getAllTransactions(self):
    #     transactions = self.getTransactions()
    #     nodes = transactions['nodes']

    #     while transactions['pageInfo']['hasNextPage']:
    #         transactions = self.getTransactions(after=transactions['pageInfo']['endCursor'])
    #         nodes += transactions['nodes']

    #         # page won't change..

    #     return nodes

    def cashout(self, amount: float, currency: str='EUR'):
        query = "mutation androidTransferMoney($sourceId: ID, $destinationId: ID!,$amount: AmountInput!) { "\
                    "transferMoney(input: {sourceId: $sourceId, destinationId: $destinationId, amount: $amount}) { "\
                        "errors { message path } "\
                    "}"\
                "}"
        variables = {
            "amount": {
                "value": amount,
                "currency": currency
            },
            "sourceId": self.walletId,
            "destinationId": self.app.bankAccount.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('transferMoney') or response.get('transferMoney', {}).get('errors'):
            errors = response.get('errors', [])
            errors += response.get('transferMoney', {}).get('errors', [])

            for error in errors:
                if error['message'] == "Source doesn't have enough funds":
                    raise kard.exceptions.InsufficientFundsException('Insufficient funds')
                elif error['message'].startswith("Variable $amount of type AmountInput! was provided invalid value"):
                    raise kard.exceptions.InvalidValueException(error['message'])

            raise kard.exceptions.GraphQLException('Expected "transferMoney" to be present. Got: ' + str(response))

        return response['transferMoney']

    def transferBalance(self, amount: float, currency: str='EUR'):
        return self.cashout(amount=amount, currency=currency)

    def cashoutAll(self, maximum: float=None):
        if not maximum:
            maximum = self.balance['value']
        else:
            maximum = min(maximum, self.balance['value'])

        return self.cashout(amount=maximum)

    def transferAllBalance(self, maximum: float=None):
        return self.cashoutAll(maximum=maximum)



from ...app import Kard
from ... import kard

class KardVaults:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidListVault { "\
                    "me { vaults { ... Vault_VaultParts } }"\
                "}\n\n"\
                "fragment Vault_VaultParts on Vault { id name color emoji { name unicode } goal { value } balance { value }}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('vaults') and not isinstance(response['me']['vaults'], list):
            raise kard.exceptions.GraphQLException('Expected "me.vaults" to be present. Got: ' + str(response))

        return response['me']['vaults']

    def getVaults(self, asObject: bool=True):
        vaults = self.getCompleteData()
        if not asObject:
            return vaults

        vaultsObj = []

        for vault in vaults:
            vaultsObj.append(self.getVault(vault['id']))

        return vaultsObj

    def getVault(self, vaultId: str):
        for vault in self.getVaults(asObject=False):
            if vault['id'] == vaultId:
                return self.vault._new(vaultId)

        raise kard.exceptions.VaultNotFoundException("Vault %s not found" % vaultId)


    def createVault(self, name: str, goal: float, currency: str='EUR'):
        query = "mutation androidCreateVault($goal: AmountInput!, $name: Name!) { "\
                    "createVault(input: {goal: $goal, name: $name}) { "\
                        "errors { message path } vault { id } "\
                    "}"\
                "}"
        variables = {
            "name": name,
            "goal": {
                "value": goal,
                "currency": currency
            }
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('createVault', {}).get('vault', {}).get('id'):
            raise kard.exceptions.GraphQLException('Expected "createVault.vault.id" to be present. Got: ' + str(response))

        return self.getVault(response['createVault']['vault']['id'])

    def deleteVault(self, vaultId: str):
        vault = self.getVault(vaultId)
        return vault.delete()



from ...app import Kard
from ... import kard

import requests

class KardCards:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Card_CardParts on Card { __typename id activatedAt customText name visibleNumber blocked expirationAt "\
                "scheme ... on PhysicalCard { atm contactless swipe online design reorderExpiredFees { value currency { isoCode } } "\
                    "orderedAt orderDelayed isMain isSoonExpired estimatedCraftingDate estimatedDeliveryDate "\
                "}"\
            "}\n\n"\
            "fragment Me_MeParts on Me { "\
                "card { id } cardBeingSetup { __typename id customText name ... on PhysicalCard { design } } "\
                "cards { nodes { ... Card_CardParts } } "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('cards', {}).get('nodes'):
            raise kard.exceptions.GraphQLException('Expected "me.cards.nodes" to be present. Got: ' + str(response))

        return response['me']['cards']['nodes']

    def getCard(self, cardId: str):
        for card in self.getCards():
            if card['id'] == cardId:
                return card

        raise kard.exceptions.CardNotFoundException('Card "%s" not found' % cardId)

    def getCards(self):
        return self.getCompleteData()

    def getPhysicalCards(self):
        return [card for card in self.getCards() if card['__typename'] == 'PhysicalCard']

    def getVirtualCards(self):
        return [card for card in self.getCards() if card['__typename'] == 'VirtualCard']

    def getPIN(self, cardId: str):
        query = "query androidUrlToGetPin($cardId: ID!) { urlToGetPin(cardId: $cardId) { ip url }}"
        variables = {
            "cardId": cardId
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('urlToGetPin', {}).get('url'):
            raise kard.exceptions.GraphQLException('Expected "urlToGetPin.url" to be present. Got: ' + str(response))

        response = requests.request(
            'GET',
            response['urlToGetPin']['url'],
            headers={
                "Authorization": "Bearer " + self.app.settings.getAccessToken(),
                "VendorIdentifier": self.app.settings.getVendorIdentifier()
            }
        )
        if response.status_code != 200 or not response.json().get('card_pin'):
            raise kard.exceptions.GraphQLException('Expected "card_pin" to be present. Got: ' + str(response.content))
        
        return response.json()['card_pin']

    def getPin(self, cardId: str):
        return self.getPIN(cardId)

    def getDigits(self, cardId: str):
        query = "query androidUrlToGetPan($cardId: ID!) { urlToGetPan(cardId: $cardId) { ip url }}"
        variables = {
            "cardId": cardId
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('urlToGetPan', {}).get('url'):
            raise kard.exceptions.GraphQLException('Expected "urlToGetPan.url" to be present. Got: ' + str(response))
        
        response = requests.request(
            'GET',
            response['urlToGetPan']['url'],
            headers={
                "Authorization": "Bearer " + self.app.settings.getAccessToken(),
                "VendorIdentifier": self.app.settings.getVendorIdentifier()
            }
        )
        if response.status_code != 200 or not response.json().get('card_pan'):
            raise kard.exceptions.GraphQLException('Expected "card_pan" to be present. Got: ' + str(response.content))
        
        return response.json()

    def getNumbers(self, cardId: str):
        return self.getDigits(cardId)

    def getDetails(self, cardId: str):
        return self.getDigits(cardId)

    def getAvailableBackgroundCustomizations(self):
        query = "query androidCardCustomizationBackgrounds { cardCustomizationBackgrounds { id name image { url } }}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('cardCustomizationBackgrounds'):
            raise kard.exceptions.GraphQLException('Expected "cardCustomizationBackgrounds" to be present. Got: ' + str(response))

        return response['cardCustomizationBackgrounds']


    def update(self, cardId: str, attributes: dict):
        query = "mutation androidUpdateCard($input: UpdateCardInput!) { "\
                    "updateCard(input: $input) { card { ... Card_CardParts } errors { path message} }}\n\n"\
                "fragment Card_CardParts on Card { "\
                    "__typename id activatedAt customText name visibleNumber blocked expirationAt "\
                    "scheme ... on PhysicalCard { atm contactless swipe online design reorderExpiredFees { value currency { isoCode } } "\
                    "orderedAt orderDelayed isMain isSoonExpired estimatedCraftingDate estimatedDeliveryDate "\
                "}"\
            "}"
        variables = {
            "input": {
                "cardId": cardId,
                "attributes": attributes
            }
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('updateCard', {}).get('card'):
            raise kard.exceptions.GraphQLException('Expected "updateCard.card" to be present. Got: ' + str(response))

        return response['updateCard']['card']

    def block(self, cardId: str):
        return self.update(cardId, {"blocked": True})
    
    def unblock(self, cardId: str):
        return self.update(cardId, {"blocked": False})



from ...app import Kard
from ... import kard

class KardBeneficiaries:
    def __init__(self, kard: Kard):
        self.app = kard

    def getAll(self):
        query = "query androidListBeneficiaries { me { "\
                    "beneficiaries { bankLogo { url } bankName iban bic id surname fullName } "\
                "}}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('beneficiaries'):
            raise kard.exceptions.GraphQLException('Expected "me.beneficiaries" to be present. Got: ' + str(response))
        
        beneficiaries = []
        for beneficiary in response['me']['beneficiaries']:
            beneficiaries.append( beneficiary )

        return beneficiaries

    def get(self, beneficiaryId: str):
        _all = self.getAll()
        for beneficiary in _all:
            if beneficiary['id'] == beneficiaryId:
                return self.beneficiary._new(beneficiary.pop('id'), **beneficiary)

        raise kard.exceptions.BeneficiaryNotFoundError("Beneficiary %s not found" % beneficiaryId)



import requests

class GraphQL:
    def __init__(self, host: str, path: str, vendorIdentifier: str):
        self.host = host
        self.path = path
        self.url = 'https://' + self.host + self.path

        self.s = requests.Session()
        self.s.headers.update({
            'Accept-Encoding': 'gzip',
            'Accept-Language': 'en',
            'Content-Type': 'application/json',
            'Host': self.host,
            'User-Agent': 'okhttp/4.9.3',
            'vendorIdentifier': vendorIdentifier,
        })

    def setVendorIdentifier(self, vendorIdentifier: str):
        self.s.headers.update({
            'vendorIdentifier': vendorIdentifier,
        })

    def setAccessToken(self, accessToken: str):
        self.s.headers.update({
            'Authorization': 'Bearer ' + accessToken,
        })


    def request(self, query, variables: dict={}, extensions: dict={}):
        response = self.s.post(
            self.url,
            json={
                'query': query,
                'variables': variables,
                'extensions': extensions
            }
        )
        if response.status_code == 200:
            r = response.json()
            if r.get('data'):
                return r['data']
            return r
        else:
            raise Exception('Query failed to run by returning code of {}.\nQuery: {}\nResponse: {}'.format(response.status_code, query, response.content))


from ...app import Kard
from ... import kard

class KardVault:
    def __init__(self, kard: Kard):
        self.app = kard

    def _new(self, vaultId: str):
        vault = KardVault(self.app)
        vault._setId(vaultId)
        return vault

    def _setId(self, vaultId: str):
        self._id = vaultId

    def _getId(self):
        return self._id


    def getCompleteData(self):
        query = "query getVault($vaultId: ID!) { vault(vaultId: $vaultId) { "\
            "id name color emoji { name unicode } goal { value currency { symbol isoCode } } "\
            "balance { value currency { symbol isoCode } }"\
        "} }"
        variables = {
            "vaultId": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('vault'):
            raise kard.exceptions.GraphQLException('Expected "vault" to be present. Got: ' + str(response))

        return response['vault']

    @property
    def id(self):
        return self._getId()

    @property
    def name(self):
        return self.getName()

    @property
    def balance(self):
        return self.getBalance()
    
    @property
    def goal(self):
        return self.getGoal()
    
    @property
    def emoji(self):
        return self.getEmoji()
    
    @property
    def color(self):
        return self.getColor()
    
    @property
    def colour(self):
        return self.color


    def getName(self):
        query = "query getVault($vaultId: ID!) { vault(vaultId: $vaultId) { name } }"
        variables = {
            "vaultId": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('vault', {}).get('name'):
            raise kard.exceptions.GraphQLException('Expected "vault.name" to be present. Got: ' + str(response))

        return response['vault']['name']

    def getBalance(self):
        query = "query getVault($vaultId: ID!) { vault(vaultId: $vaultId) { balance { value currency { symbol isoCode } } } }"
        variables = {
            "vaultId": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('vault', {}).get('balance'):
            raise kard.exceptions.GraphQLException('Expected "vault.balance" to be present. Got: ' + str(response))

        return response['vault']['balance']

    def getGoal(self):
        query = "query getVault($vaultId: ID!) { vault(vaultId: $vaultId) { goal { value currency { symbol isoCode } } } }"
        variables = {
            "vaultId": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('vault', {}).get('goal'):
            raise kard.exceptions.GraphQLException('Expected "vault.goal" to be present. Got: ' + str(response))

        return response['vault']['goal']
    
    def getEmoji(self):
        query = "query getVault($vaultId: ID!) { vault(vaultId: $vaultId) { emoji { name unicode } } }"
        variables = {
            "vaultId": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('vault', {}).get('emoji'):
            raise kard.exceptions.GraphQLException('Expected "vault.emoji" to be present. Got: ' + str(response))

        return response['vault']['emoji']

    def getColor(self):
        query = "query getVault($vaultId: ID!) { vault(vaultId: $vaultId) { color } }"
        variables = {
            "vaultId": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('vault', {}).get('color'):
            raise kard.exceptions.GraphQLException('Expected "vault.color" to be present. Got: ' + str(response))

        return response['vault']['color']
    
    def getColour(self):
        return self.getColor()

    def getTransactions(self, first: int=10, after: str=None, numberOfComments: int=3):
        query = "query androidGetVaultTransactions($vaultId: ID!, $first: Int, $after: String, $numberOfComments: Int) { "\
                    "vault(vaultId: $vaultId) { id transactions(first: $first, after: $after) { "\
                        "pageInfo { endCursor hasNextPage } nodes { ... Transaction_TransactionParts } "\
                    "} }"\
                "}\n\n"\
                "fragment Transaction_TransactionParts on Transaction { "\
                    "__typename id title status address image { id url } visibility amount { value currency { symbol } } "\
                    "category { name color image { url } } processedAt unreadCommentsCount lastCommentReadAt "\
                    "comments(first: $numberOfComments) { "\
                        "totalCount pageInfo { endCursor hasNextPage } "\
                            "nodes { id comment createdAt user { id nickname firstName lastName avatar { url } } } "\
                    "} quickAnswers { message } user { id firstName lastName username avatar { url } } "\
                "}"
        variables = {
            "vaultId": self._getId(),
            "first": first,
            "after": after,
            "numberOfComments": numberOfComments
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('vault', {}).get('transactions'):
            raise kard.exceptions.GraphQLException('Expected "vault.transactions" to be present. Got: ' + str(response))
        
        return response['vault']['transactions']


    def setName(self, name: str):
        query = "mutation androidUpdateVault($vaultId: ID!, $name: Name) { "\
                "updateVault(input: {vaultId: $vaultId, name: $name}) { "\
                    "errors { message path } vault { id name } "\
                "}"\
            "}"
        variables = {
            "vaultId": self.id,
            "name": name
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('updateVault', {}).get('vault'):
            raise kard.exceptions.GraphQLException('Expected "updateVault.vault" to be present. Got: ' + str(response))
        
        return response['updateVault']

    def setEmoji(self, emoji: str):
        valid_emojies = ["🎁", "🎈", "🛍", "💰", "😈", "🎓", "🏝", "🎫", "🎸", "✈️", "👟", "📱", "🎮", "🛴", "🛵"]

        query = "mutation androidUpdateVault($vaultId: ID!, $emoji: EmojiInput) { "\
                "updateVault(input: {vaultId: $vaultId, emoji: $emoji}) { "\
                    "errors { message path } vault { id emoji { name unicode } } "\
                "}"\
            "}"
        variables = {
            "vaultId": self.id,
            "emoji": emoji
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('updateVault', {}).get('vault'):
            raise kard.exceptions.GraphQLException('Expected "updateVault.vault" to be present. Got: ' + str(response))
        
        return response['updateVault']

    def setColor(self, color: str):
        official_colors = {
            "purpleblack": "#1f193f",    "grey":"#75818c",
            "purplelight": "#bd3fdd",    "yellow":"#ffca10",
            "purpledark": "#9850ff",     "pink":"#f943b1",
            "black": "#1b1d20",          "green":"#3ce977",
            "orange":"#ff9455",          "red": "#ff5f7c",
            "cyan":"#15e4da",            "blue": "#35c4ff"
        } # Any hexadecimal code will work though! (Must provide in #xxxxxx format)
          
        if color in official_colors:
            color = official_colors[color]

        query = "mutation androidUpdateVault($vaultId: ID!, $color: HexadecimalColorCode) { "\
                "updateVault(input: {vaultId: $vaultId, color: $color}) { "\
                    "errors { message path } vault { id color } "\
                "}"\
            "}"
        variables = {
            "vaultId": self.id,
            "color": color
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('updateVault', {}).get('vault'):
            raise kard.exceptions.GraphQLException('Expected "updateVault.vault" to be present. Got: ' + str(response))
        
        return response['updateVault']

    def setColour(self, colour: str):
        return self.setColor(colour)


    def topup(self, amount: float, currency: str="EUR"):
        query = "mutation androidTransferMoney($sourceId: ID, $destinationId: ID!,$amount: AmountInput!) { "\
                    "transferMoney(input: {sourceId: $sourceId, destinationId: $destinationId, amount: $amount}) { "\
                        "errors { message path } "\
                    "}"\
                "}"
        variables = {
            "destinationId": self.id,
            "sourceId": self.app.bankAccount.id,
            "amount": {
                "value": amount,
                "currency": currency
            }
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('transferMoney'):
            raise kard.exceptions.GraphQLException('Expected "transferMoney" to be present. Got: ' + str(response))
        
        return response['transferMoney']

    def withdraw(self, amount: float, currency: str="EUR"):
        query = "mutation androidTransferMoney($sourceId: ID, $destinationId: ID!,$amount: AmountInput!) { "\
                    "transferMoney(input: {sourceId: $sourceId, destinationId: $destinationId, amount: $amount}) { "\
                        "errors { message path } "\
                    "}"\
                "}"
        variables = {
            "destinationId": self.app.bankAccount.id,
            "sourceId": self.id,
            "amount": {
                "value": amount,
                "currency": currency
            }
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('transferMoney'):
            raise kard.exceptions.GraphQLException('Expected "transferMoney" to be present. Got: ' + str(response))
        
        return response['transferMoney']

    def delete(self):
        query = "mutation androidCloseVault($vaultId: ID!) { closeVault(input: {vaultId: $vaultId}) { "\
                    "errors { message path } }"\
                "}"
        variables = {
            "vaultId": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('closeVault'):
            raise kard.exceptions.GraphQLException('Expected "closeVault" to be present. Got: ' + str(response))
        
        return response['closeVault']




from ...app import Kard
from ... import kard

class KardBank:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Me_MeParts on Me { "\
                "bankAccount { "\
                    "id iban bic user { firstName lastName } balance { value currency { symbol isoCode } } "\
                    "lockedTransactions createdAt"\
                "} "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('bankAccount'):
            raise kard.exceptions.GraphQLException('Expected "me.bankAccount" to be present. Got: ' + str(response))

        return response['me']['bankAccount']

    @property
    def id(self):
        return self.getId()
    
    @property
    def iban(self):
        return self.getIBAN()
    
    @property
    def bic(self):
        return self.getBIC()
    
    @property
    def user(self):
        return self.getUser()

    @property
    def owner(self):
        return self.user
    
    @property
    def balance(self):
        return self.getBalance()

    def getId(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Me_MeParts on Me { "\
                "bankAccount { "\
                    "id "\
                "} "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('bankAccount', {}).get('id'):
            raise kard.exceptions.GraphQLException('Expected "me.bankAccount.id" to be present. Got: ' + str(response))
        
        return response['me']['bankAccount']['id']

    def getIBAN(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Me_MeParts on Me { "\
                "bankAccount { "\
                    "iban "\
                "} "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('bankAccount', {}).get('iban'):
            raise kard.exceptions.GraphQLException('Expected "me.bankAccount.iban" to be present. Got: ' + str(response))
        
        return response['me']['bankAccount']['iban']
    
    def getIban(self):
        return self.getIBAN()

    def getBIC(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Me_MeParts on Me { "\
                "bankAccount { "\
                    "bic "\
                "} "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('bankAccount', {}).get('bic'):
            raise kard.exceptions.GraphQLException('Expected "me.bankAccount.bic" to be present. Got: ' + str(response))
        
        return response['me']['bankAccount']['bic']

    def getBic(self):
        return self.getBIC()

    def getUser(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Me_MeParts on Me { "\
                "bankAccount { "\
                    "user { firstName lastName } "\
                "} "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('bankAccount', {}).get('user'):
            raise kard.exceptions.GraphQLException('Expected "me.bankAccount.user" to be present. Got: ' + str(response))
        
        return response['me']['bankAccount']['user']

    def getOwner(self):
        return self.getUser()

    def getBalance(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Me_MeParts on Me { "\
                "bankAccount { "\
                    "balance { value currency { symbol isoCode } } "\
                "} "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('bankAccount', {}).get('balance'):
            raise kard.exceptions.GraphQLException('Expected "me.bankAccount.balance" to be present. Got: ' + str(response))
        
        return response['me']['bankAccount']['balance']
    
    def getLockedTransactions(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Me_MeParts on Me { "\
                "bankAccount { "\
                    "lockedTransactions "\
                "} "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('bankAccount', {}).get('lockedTransactions'):
            raise kard.exceptions.GraphQLException('Expected "me.bankAccount.lockedTransactions" to be present. Got: ' + str(response))
        
        return response['me']['bankAccount']['lockedTransactions']
    
    def getCreatedAt(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Me_MeParts on Me { "\
                "bankAccount { "\
                    "createdAt "\
                "} "\
            "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('bankAccount', {}).get('createdAt'):
            raise kard.exceptions.GraphQLException('Expected "me.bankAccount.createdAt" to be present. Got: ' + str(response))
        
        return response['me']['bankAccount']['createdAt']


    def sendMoneyToUser(self, userId: str, amount: float, currency: str='EUR', reason: str=''):
        query = "mutation androidSendMoney($input: SendMoneyInput!) { sendMoney(input: $input) { errors { path message } } }"
        variables = {
            "input": {
                "internalUsersIds": [userId],
                "externalUsers": [],
                "amount": {"value": amount, "currency": currency},
                "reason": reason
            }
		}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('sendMoney'):
            raise kard.exceptions.GraphQLException('Expected "sendMoney" to be present. Got: ' + str(response))
        
        if response['sendMoney'].get('errors'):
            raise kard.exceptions.GraphQLException('Expected "sendMoney.errors" to be empty. Got: ' + str(response))

        return True

